@@ -1,5 +1,5 @@
 # mt-aws-glacier - Amazon Glacier sync client
-# Copyright (C) 2012-2014  Victor Efimov
+# Copyright (C) 2012-2013  Victor Efimov
 # http://mt-aws.com (also http://vs-dev.com) vs@vs-dev.com
 # License: GPLv3
 #
@@ -29,9 +29,7 @@ use App::MtAws::Journal;
 use File::Path;
 use App::MtAws::TreeHash;
 use Test::More;
-use Digest::SHA qw/sha256_hex/;
 use Encode;
-use File::stat;
 use Carp;
 
 
@@ -42,7 +40,6 @@ sub new
 	defined($self->{create_journal_version})||confess;
 	$self->{filenames_encoding} ||= 'UTF-8';
 	$self->{journal_encoding} ||= 'UTF-8';
-	$self->{absdataroot} = $self->{dataroot};
 	bless $self, $class;
 	return $self;
 }
@@ -50,137 +47,89 @@ sub new
 sub test_all
 {
 	my ($self) = @_;
-
-	$self->test(q{test_journal});
-	$self->test(q{test_real_files});
-
-	$self->test(q{test_all_files});
-	$self->test(q{test_new_files});
-	$self->test(q{test_existing_files});
-}
-
-sub binarypath
-{
-	my ($self, $path) = @_;
-	encode($self->{filenames_encoding}, $path, Encode::DIE_ON_ERR|Encode::LEAVE_SRC)
-}
-
-sub characterpath
-{
-	my ($self, $path) = @_;
-	decode($self->{filenames_encoding}, $path, Encode::DIE_ON_ERR|Encode::LEAVE_SRC)
-}
-
-sub test
-{
-	my ($self, $testname) = @_;
-
-	my $pwd  = Cwd::getcwd();
-
-	mkpath($self->binarypath($self->{absdataroot}));;
-	die unless -d $self->binarypath($self->{absdataroot});
-
-	if (defined ($self->{curdir})) {
-		mkpath $self->binarypath($self->{curdir});
-		$self->{dataroot} = File::Spec->abs2rel($self->{absdataroot}, $self->{curdir});
-		chdir $self->binarypath($self->{curdir}) or die "[$self->{dataroot}\t$self->{curdir}]";
-	} else {
-		chdir $self->binarypath($self->{mtroot}) or die;
-	}
-
-	$self->$testname();
-
-
-	chdir $self->binarypath($pwd) or die;
-	-d && rmtree($_) for ($self->binarypath($self->{tmproot}));
-}
-
-sub check_absfilename
-{
-	my ($self, $should_exist, $relfilename, $absfilename) = @_;
-
-	my $absfilename_old = $self->characterpath(File::Spec->rel2abs($self->binarypath($relfilename), $self->binarypath($self->{dataroot})));
-	my $absfilename_correct = File::Spec->catfile($self->{dataroot}, $relfilename);
-	my $absfilename_wrong = $self->characterpath(File::Spec->abs2rel(File::Spec->rel2abs($self->binarypath($relfilename), $self->binarypath($self->{dataroot}))));
-
-	if ($should_exist) {
-		my $ino_old = stat($self->binarypath($absfilename_old))->ino;
-		ok $ino_old;
-		is stat($self->binarypath($absfilename_correct))->ino, $ino_old;
-		is stat($self->binarypath($absfilename_wrong))->ino, $ino_old;
-	}
-
-	#TODO: add File::Spec->canonpath() to _correct and fix absfilename_correct=./dirA/file3
-	ok $absfilename_old =~ m{^/};
-
-	ok $absfilename !~ m{//};
-
-	ok $absfilename =~ m{^\Q$self->{dataroot}/\E} unless $self->{dataroot} =~ m{^\.(/|$)};
-	is ($absfilename, $absfilename_correct, "absfilename match");
+	$self->test_journal;
+	$self->test_real_files;
+	
+	$self->test_all_files;
+	$self->test_new_files;
+	$self->test_existing_files;
 }
 
 sub test_journal
 {
 	my ($self) = @_;
+	mkpath(encode($self->{filenames_encoding}, $self->{dataroot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC));
 	$self->create_journal();
-
+	
 	my $j = App::MtAws::Journal->new(journal_encoding => $self->{journal_encoding},
 		journal_file => $self->{journal_file}, root_dir => $self->{dataroot}, follow => $self->{follow});
 	$j->read_journal(should_exist => 1);
-
+	
 	my @checkfiles = grep { $_->{type} eq 'normalfile' && $_->{journal} && $_->{journal} eq 'created' } @{$self->{testfiles}};
 	ok(( scalar @checkfiles == scalar keys %{ $j->{journal_h} } ), "journal - number of planed and real files match");
-
+	
 	for my $cf (@checkfiles) {
 		ok (my $jf = $j->{journal_h}->{$cf->{filename}}, "file $cf->{filename} exists in Journal");
 		ok ($jf->{size} == $cf->{filesize}, "file size match $jf->{size} == $cf->{filesize}");
 		ok ($jf->{treehash} eq $cf->{final_hash}, "treehash matches");
 		ok ($jf->{archive_id} eq $cf->{archive_id}, "archive id matches");
-		$self->check_absfilename(0, $cf->{filename}, $j->absfilename($cf->{filename}));
+		is ($j->absfilename($cf->{filename}), File::Spec->rel2abs($cf->{filename}, $self->{dataroot}), "absfilename match"); # actually better test in real
 	}
+	
+	my $tmproot_e = encode($self->{filenames_encoding}, $self->{tmproot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);
+	rmtree($tmproot_e)
+		if ($tmproot_e) && (-d $tmproot_e);
 }
 
 sub test_real_files
 {
 	my ($self) = @_;
+	mkpath(encode($self->{filenames_encoding}, $self->{dataroot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC));
 	$self->create_files();
-
+	
 	my $j = App::MtAws::Journal->new(journal_encoding => $self->{journal_encoding},
 		journal_file => $self->{journal_file}, root_dir => $self->{dataroot}, filter => $self->{filter}, follow => $self->{follow});
 	$j->read_files({new=>1,existing=>1});
-
+	
 	my @checkfiles = grep { $_->{type} ne 'dir' && !$_->{exclude} } @{$self->{testfiles}};
 	ok((scalar @checkfiles) == scalar @{$j->{listing}{new}}+scalar @{$j->{listing}{existing}}, "number of planed and real files match");
-
+	
 	my %testfile_h = map { $_->{filename } => $_} @checkfiles;
 	for my $realfile (@{$j->{listing}{new}}, @{$j->{listing}{existing}}) {
 		ok ( $testfile_h{ $realfile->{relfilename} }, "found file $realfile->{relfilename} exists in planned test file list" );
-		$self->check_absfilename(1, $realfile->{relfilename}, $j->absfilename($realfile->{relfilename}));
 	}
+	my $tmproot_e = encode($self->{filenames_encoding}, $self->{tmproot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);
+	rmtree($tmproot_e)
+		if ($tmproot_e) && (-d $tmproot_e);
 }
 
 sub test_all_files
 {
 	my ($self) = @_;
+	mkpath(encode($self->{filenames_encoding}, $self->{dataroot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC));
 	$self->create_journal();
 	$self->create_files('skip');
 	my $j = App::MtAws::Journal->new(journal_encoding => $self->{journal_encoding},
 		journal_file => $self->{journal_file}, root_dir => $self->{dataroot}, filter => $self->{filter}, follow => $self->{follow});
 	$j->read_files({new=>1,existing=>1});
-
+	
 	my @checkfiles = grep { $_->{type} ne 'dir' && !$_->{skip} && !$_->{exclude} } @{$self->{testfiles}};
 	ok((scalar @checkfiles) == scalar @{$j->{listing}{new}}+scalar @{$j->{listing}{existing}}, "number of planed and real files match");
-
+	
 	my %testfile_h = map { $_->{filename } => $_} @checkfiles;
 	for my $realfile (@{$j->{listing}{new}}, @{$j->{listing}{existing}}) {
 		ok ( $testfile_h{ $realfile->{relfilename} }, "found file $realfile->{relfilename} exists in planned test file list" );
 	}
+	my $tmproot_e = encode($self->{filenames_encoding}, $self->{tmproot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);
+	rmtree($tmproot_e)
+		if ($tmproot_e) && (-d $tmproot_e);
 }
 
 
 sub test_new_files
 {
 	my ($self) = @_;
+	mkpath(encode($self->{filenames_encoding}, $self->{dataroot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC));
 	$self->create_journal();
 	$self->create_files('skip');
 	my $j = App::MtAws::Journal->new(journal_encoding => $self->{journal_encoding},
@@ -195,32 +144,39 @@ sub test_new_files
 	for my $realfile (@{$j->{listing}{new}}) {
 		ok ( $testfile_h{ $realfile->{relfilename} }, "found file $realfile->{relfilename} exists in planned test file list" );
 	}
+	my $tmproot_e = encode($self->{filenames_encoding}, $self->{tmproot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);
+	rmtree($tmproot_e)
+		if ($tmproot_e) && (-d $tmproot_e);
 }
 
 sub test_existing_files
 {
 	my ($self) = @_;
+	mkpath(encode($self->{filenames_encoding}, $self->{dataroot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC));
 	$self->create_journal();
 	$self->create_files('skip');
 	my $j = App::MtAws::Journal->new(journal_encoding => $self->{journal_encoding},
 		journal_file => $self->{journal_file}, root_dir => $self->{dataroot}, filter => $self->{filter}, follow => $self->{follow});
 	$j->read_journal(should_exist => 1);
 	$j->read_files({existing=>1});
-
+	
 	my @checkfiles = grep { $_->{type} ne 'dir' && !$_->{skip} && !$_->{exclude} && ($_->{journal} && $_->{journal} eq 'created')} @{$self->{testfiles}};
 	ok((scalar @checkfiles) == scalar @{$j->{listing}{existing}}, "number of planed and real files match");
-
+	
 	my %testfile_h = map { $_->{filename } => $_} @checkfiles;
 	for my $realfile (@{$j->{listing}{existing}}) {
 		ok ( $testfile_h{ $realfile->{relfilename} }, "found file $realfile->{relfilename} exists in planned test file list" );
 	}
+	my $tmproot_e = encode($self->{filenames_encoding}, $self->{tmproot}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);
+	rmtree($tmproot_e)
+		if ($tmproot_e) && (-d $tmproot_e);
 }
 
 sub create_files
 {
 	my ($self, $mode) = @_;
 	for my $testfile (@{$self->{testfiles}}) {
-		$testfile->{fullname} = "$self->{absdataroot}/$testfile->{filename}";
+		$testfile->{fullname} = "$self->{dataroot}/$testfile->{filename}";
 		if ($testfile->{type} eq 'dir') {
 			mkpath(encode($self->{filenames_encoding}, $testfile->{fullname}, Encode::DIE_ON_ERR|Encode::LEAVE_SRC));
 		} elsif (($testfile->{type} eq 'normalfile')  && ( #&& !$testfile->{exclude}
@@ -299,27 +255,24 @@ sub create_journal_vABC
 	close F;
 }
 
-our $global_cnt;
-
 sub get_random_archive_id
 {
 	my ($i) = @_;
-	$global_cnt++;
-	sha256_hex($$.time().$i.$global_cnt);
+	my $th = App::MtAws::TreeHash->new();
+	my $s = $$.time().$i;
+	$th->eat_data(\$s);
+	$th->calc_tree();
+	$th->get_final_hash();
 }
 
-our %treehash_cache;
-
 sub scalar_treehash
 {
 	my ($str) = @_;
 	confess if utf8::is_utf8($str);
-	$treehash_cache{$str} ||= do {
-		my $th = App::MtAws::TreeHash->new();
-		$th->eat_data(\$str);
-		$th->calc_tree();
-		$th->get_final_hash();
-	}
+	my $th = App::MtAws::TreeHash->new();
+	$th->eat_data(\$str);
+	$th->calc_tree();
+	$th->get_final_hash();
 }
 
 1;
